﻿using System.Globalization;
using System.Reflection;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using System.Text;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using System.Data;
using System;
using Produmex.Sbo.Logex.Redrock.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using System.Collections.Generic;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.SqlClient;

// PmxWorkflowExecutionTypes.HOOK_FLOW
namespace WorkflowScript_SalesDeliveryCreatedHookScript
{
    /// <summary>
    /// This is a hook script, meant to be customized on a "per project" basis
    /// </summary>
    public class WorkflowScript_SalesDeliveryCreatedHookScript : WorkflowInstanceScriptBase
    {
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        // Input parameters * do not change *
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;
        public ReadOnlyBinder<DocumentRef[]> DocumentRefs;

        // Output parameters
        // <none>

        // Sub-flows
        // <none>

        /// <summary>
        /// Initializes a new instance of the <see cref="WorkflowScript_SalesDeliveryCreatedHookScript"/> class.
        /// </summary>
        /// <param name="parent">The parent.</param>
        /// <param name="factory">The factory.</param>
        public WorkflowScript_SalesDeliveryCreatedHookScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {
        }

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
            // Parameters in scope
            Session session = GetScopeParameter("Session") as Session;
            ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;

            List<DeliveredSSCCInfo> infos = new List<DeliveredSSCCInfo>();

            DocumentRef[] docRefs = DocumentRefs.Get();
            string query = BuildQuery.GetAllSSCCInfo(docRefs);
            DataRow[] rows = sboProviderService.RunView(true, null, null, query).Tables[0].Select();
            s_log.Warn(query);

            foreach (DataRow row in rows)
            {
                DeliveredSSCCInfo info = new DeliveredSSCCInfo
                {
                    DocEntry = Convert.ToInt32(row[PmxSalesDeliveryNoteTable.Columns.DocEntry.NAME]),
                    DocNum = Convert.ToInt32(row[PmxSalesDeliveryNoteTable.Columns.DocNum.NAME]),
                    Luid = Convert.ToInt32(row[PmxLogisticUnitIDTable.Columns.InternalKey.NAME]),
                    SSCC = row[PmxLogisticUnitIDTable.Columns.SSCC.NAME].ToString(),
                    Status = PmxLUIDStatusColumnValuesRR.New.VALUE,
                    ObjectType = SboObjectTypes.SalesDelivery
                };
                infos.Add(info);
            }


            sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, new DelegateWithPmxDbConnection<object>(
                       delegate(PmxDbConnection conn, object[] parameters)
                       {
                           query = BuildQuery.InsertDeliveredSSCC(infos, conn.InternalKeySboUser);
                           SboRecordsetHelper.RunQueryNoReturn(s_log, query, conn);

                           return null;
                       }));

            return;
        }

        #endregion


    }

    #region BUILDQUERY

    public static class BuildQuery
    {
        /// <summary>
        /// Inserts the reserved lui das shipped.
        /// </summary>
        /// <param name="infos">The infos.</param>
        /// <param name="userSign">The user sign.</param>
        /// <returns></returns>
        internal static string InsertDeliveredSSCC(List<DeliveredSSCCInfo> infos, int userSign)
        {

            StringBuilder strBuilder = new StringBuilder();

            strBuilder.Append("INSERT INTO ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxDeliveredSSCCRRTable.NAME));
            strBuilder.Append("\r\n ( ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxDeliveredSSCCRRTable.Columns.Canceled.NAME));
            strBuilder.Append(", ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxDeliveredSSCCRRTable.Columns.UserSign.NAME));
            strBuilder.Append(", ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxDeliveredSSCCRRTable.Columns.CreateDateTime.NAME));
            strBuilder.Append(", ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxDeliveredSSCCRRTable.Columns.UpdateDateTime.NAME));
            strBuilder.Append(", ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxDeliveredSSCCRRTable.Columns.Version.NAME));
            strBuilder.Append(", ");

            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxDeliveredSSCCRRTable.Columns.BaseEntry.NAME));
            strBuilder.Append(", ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxDeliveredSSCCRRTable.Columns.BaseDocNum.NAME));
            strBuilder.Append(", ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxDeliveredSSCCRRTable.Columns.BaseType.NAME));
            strBuilder.Append(", ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxDeliveredSSCCRRTable.Columns.LUID.NAME));
            strBuilder.Append(", ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxDeliveredSSCCRRTable.Columns.SSCC.NAME));
            strBuilder.Append(", ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxDeliveredSSCCRRTable.Columns.Status.NAME));
            strBuilder.Append(" )");
            strBuilder.AppendFormat("\r\n VALUES ");


            for (int i = 0; i < infos.Count; i++)
            {
                DeliveredSSCCInfo info = infos[i];
                strBuilder.Append("( ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue("N", CultureInfo.InvariantCulture));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(userSign, CultureInfo.InvariantCulture));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(DateTime.Now, CultureInfo.InvariantCulture));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(DateTime.Now, CultureInfo.InvariantCulture));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(1, CultureInfo.InvariantCulture));
                strBuilder.Append(", ");

                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(info.DocEntry, CultureInfo.InvariantCulture));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(info.DocNum, CultureInfo.InvariantCulture));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(info.ObjectType, CultureInfo.InvariantCulture));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(info.Luid, CultureInfo.InvariantCulture));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(info.SSCC, CultureInfo.InvariantCulture));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(info.Status, CultureInfo.InvariantCulture));

                if (infos.Count == i + 1)
                {
                    strBuilder.Append(") ");
                }
                else
                {
                    strBuilder.Append("), \r\n  ");
                }

            }


            return strBuilder.ToString();
        }


        internal static string GetAllSSCCInfo(DocumentRef[] docRefs)
        {
            StringBuilder strBuilder = new StringBuilder();

            strBuilder.AppendFormat("SELECT DISTINCT {0}, {1}, {2}, {3}",
                 SqlCommandHelper.EscapeTableColumnName(PmxSalesDeliveryNoteTable.NAME, PmxSalesDeliveryNoteTable.Columns.DocEntry.NAME),
                  SqlCommandHelper.EscapeTableColumnName(PmxSalesDeliveryNoteTable.NAME, PmxSalesDeliveryNoteTable.Columns.DocNum.NAME),
                 SqlCommandHelper.EscapeTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME),
          SqlCommandHelper.EscapeTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.SSCC.NAME)
         );

            strBuilder.AppendFormat("\r\n FROM {0}",
         SqlCommandHelper.EscapeTableName(PmxSalesDeliveryNoteTable.NAME));

            strBuilder.AppendFormat("\r\n INNER JOIN {0} ON {1} = {2}",
               SqlCommandHelper.EscapeTableName(PmxSalesDeliveryNoteLineTable.NAME),
               SqlCommandHelper.EscapeTableColumnName(PmxSalesDeliveryNoteLineTable.NAME, PmxSalesDeliveryNoteLineTable.Columns.DocEntry.NAME),
               SqlCommandHelper.EscapeTableColumnName(PmxSalesDeliveryNoteTable.NAME, PmxSalesDeliveryNoteTable.Columns.DocEntry.NAME));

            strBuilder.AppendFormat("\r\n INNER JOIN {0} ON {1} = {2} AND {3} = {4} AND {5} = {6}",
                 SqlCommandHelper.EscapeTableName(PmxInventoryDetailTable.NAME),
                 SqlCommandHelper.EscapeTableColumnName(PmxSalesDeliveryNoteLineTable.NAME, PmxSalesDeliveryNoteLineTable.Columns.DocEntry.NAME),
                 SqlCommandHelper.EscapeTableColumnName(PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.DocEntry.NAME),
                 SqlCommandHelper.EscapeTableColumnName(PmxSalesDeliveryNoteLineTable.NAME, PmxSalesDeliveryNoteLineTable.Columns.LineNum.NAME),
                 SqlCommandHelper.EscapeTableColumnName(PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.DocLineNum.NAME),
                 SqlCommandHelper.EscapeTableColumnName(PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.TransType.NAME),
                 SqlCommandHelper.FormatAndEscapeColumnValue(SboObjectTypes.SalesDelivery, CultureInfo.InvariantCulture)
                 );


            strBuilder.AppendFormat("\r\n INNER JOIN {0} ON {1} = {2}",
                 SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME),
                 SqlCommandHelper.EscapeTableColumnName(PmxInventoryDetailTable.NAME, PmxInventoryDetailTable.Columns.LogUnitIdentKey.NAME),
                 SqlCommandHelper.EscapeTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME));

            string operand = "";
            strBuilder.AppendFormat(" WHERE {0} IN (",
                 SqlCommandHelper.EscapeTableColumnName(PmxSalesDeliveryNoteTable.NAME, PmxSalesDeliveryNoteTable.Columns.DocEntry.NAME)
            );

            foreach (DocumentRef docRef in docRefs)
            {
                strBuilder.Append(operand);
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(docRef.DocEntry, CultureInfo.InvariantCulture));
                operand = ", ";
            }
            strBuilder.Append(") ");


            return strBuilder.ToString();
        }
    }

    internal class DeliveredSSCCInfo
    {
        public int DocEntry { get; set; }
        public int DocNum { get; set; }
        public string ObjectType { get; set; }
        public int Luid { get; set; }
        public string SSCC { get; set; }
        public string Status { get; set; }

    }

    #endregion
}
