﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Data.Common;
using System.Globalization;
using System.Reflection;
using System.Text;
using System.Text.RegularExpressions;
using System.Transactions;
using Produmex.Foundation;
using Produmex.Foundation.Barcode;
using Produmex.Foundation.Data;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Convertors;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Generators.BarcodeController;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.GS1;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.SlimScreen.WinGui;
using Produmex.Foundation.Utilities;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Data.ViewObjects.Definitions.Views;
using Produmex.Foundation.Wwf.Sbo.DataObjects;
using Produmex.Foundation.Messages;
using Produmex.Foundation.Data.DbClient;
using Produmex.Sbo.Logex.Redrock.BusinessObjects.Definitions.Tables;

// PmxWorkflowExecutionTypes.SUB_FLOW
namespace Produmex.Sbo.Logex.WorkflowScripts
{
    /// <summary>
    /// V4
    /// </summary>
    public class WorkflowScript_TransferSSCCSalesDeliveryScript : WorkflowInstanceScriptBase
    {
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        // Input parameters
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;


        // Sub-flows
        // <none>



        /// <summary>
        /// Initializes a new instance of the <see cref="WorkflowScript_TransferSSCCSalesDelivery" /> class.
        /// </summary>
        /// <param name="parent">The parent.</param>
        /// <param name="factory">The factory.</param>
        public WorkflowScript_TransferSSCCSalesDeliveryScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {
        }


        #region WorkflowInstanceScriptBase Members

        /// <summary>
        /// Executes this instance.
        /// </summary>
        protected override void Execute()
        {
            // Parameters in scope
            Session session = (Session)GetScopeParameter("Session");
            ISboProviderService sboProviderService = (ISboProviderService)GetScopeParameter("<WwfService>ISboService");

            string deviceID = session.DeviceInstanceId;
            string query = null;
            string initialErrorKey = null;
            Message msg = null;

            bool goBack = false;
            bool allowGoBack = false;
            bool isReceiving = false;

            int docEntry = 0;
            int luid = 0;

            string buttonKeySelectTaskBeforeOrdering = null;
            string sscc = null;

            DataRow[] selectedRow = null;
            DataSet dsSalesDeliveries = null;
            DataSet dsSSCC = null;

        Step_SelectTask:

            #region SELECT TASK

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectButtonScreen),
          DefaultCultureInfo.Get(), BuildParamCollection(
             "InitialErrorKey", initialErrorKey,
             "TitleKey", "MSG_TITLE_SELECT_TASK",
             "AllowedButtons", new string[] { "MSG_BUTTON_RR_RECEIVING", "MSG_BUTTON_RR_SHIPPING" },
             "ShowStopButton", false
             ));
            msg = WaitForMessage();

            initialErrorKey = null;

            if (msg.Name.EndsWith(".ButtonClicked"))
            {
                buttonKeySelectTaskBeforeOrdering = ExtractParameter<string>(msg.Parameters, "buttonKey");

                if (buttonKeySelectTaskBeforeOrdering == "MSG_BUTTON_RR_RECEIVING")
                {
                    isReceiving = true;
                }
                else if (buttonKeySelectTaskBeforeOrdering == "MSG_BUTTON_RR_SHIPPING")
                {
                    isReceiving = false;
                }
                goto Step_SelectSalesDeliveries;

            }
            else
            {
                return;
            }

            #endregion

        Step_SelectSalesDeliveries:

            #region SELECT SALES DELIVERY

            query = BuildQuery.GetAllSalesDeliveries(isReceiving ? PmxLUIDStatusColumnValuesRR.New.VALUE : PmxLUIDStatusColumnValuesRR.Received.VALUE);
            dsSalesDeliveries = sboProviderService.RunView(false, null, null, query);

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Retour.ISelectSalesDeliveryNoteScreen),
                   DefaultCultureInfo.Get(), BuildParamCollection(
                   "InitialErrorKey", initialErrorKey,
                   "TitleKey", "MSG_TITLE_SELECT_SALES_DELIVERY_NOTE",
                   "SalesDeliveryNoteDS", dsSalesDeliveries
                   ));
            msg = WaitForMessage();
            initialErrorKey = null;
            if (msg.Name.EndsWith(".SalesDeliveryNoteSelected"))
            {
                docEntry = ExtractParameter<int?>(msg.Parameters, "docEntry").Value;
            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
                goBack = true;
                goto Step_SelectTask;
            }
            //else if (msg.Name.EndsWith(".RefreshRequested"))
            //{
            //    goto Step_SelectSalesOrders;
            //}

            goto Step_ScanSSCC;

            #endregion

            Step_ScanSSCC:

            #region SCAN SSCC



            query = BuildQuery.GetAllSSCCs(docEntry, isReceiving ? PmxLUIDStatusColumnValuesRR.New.VALUE : PmxLUIDStatusColumnValuesRR.Received.VALUE);
            dsSSCC = sboProviderService.RunView(true, null, null, query);

            if (GetNumberOfRows(dsSSCC) == 0)
            {
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                                            DefaultCultureInfo.Get(), BuildParamCollection(
                                                "InitialErrorKey", initialErrorKey,
                                                "MessageKey", "MSG_RR_NOTHING_NEEDS_RECEPTION",
                                                "ShowButton", true));

                msg = WaitForMessage();
                goto Step_SelectSalesDeliveries;
            }

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IIdentifySSCCListScreen),
                               DefaultCultureInfo.Get(), BuildParamCollection(
                               "InitialErrorKey", initialErrorKey,
                                   "TitleKey", "MSG_TITLE_RR_SELECT_SSCC",
                                   "SSCCDS", dsSSCC
                                   ));
            msg = WaitForMessage();
            initialErrorKey = null;

            if (msg.Name.EndsWith(".SSCCEntered"))
            {
                sscc = ExtractParameter<string>(msg.Parameters, "sscc");
                selectedRow = dsSSCC.Tables[0].Select(string.Format("{0} = {1}", SSCCDataSet.Columns.SSCC.Name, sscc));

                if (selectedRow.Length == 0)
                {
                    initialErrorKey = "MSG_RR_ERROR_SSCC_NOT_FOUND";
                    sscc = null;
                    selectedRow = null;
                    goto Step_ScanSSCC;
                }

                luid = Convert.ToInt32(selectedRow[0][PmxReservedLUIDRRTable.Columns.LUID.NAME]);
                goto Step_SetData;
            }
            goBack = true;
            goto Step_SelectSalesDeliveries;

            #endregion
        Step_SetData:

            #region SET DATA


                sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, new DelegateWithPmxDbConnection<object>(
                         delegate(PmxDbConnection conn, object[] parameters)
                         {
                             query = BuildQuery.UpdateDeliveredSSCC( luid, docEntry,SboObjectTypes.SalesDelivery, isReceiving ? PmxLUIDStatusColumnValuesRR.Received.VALUE : PmxLUIDStatusColumnValuesRR.Delivered.VALUE );
                             SboRecordsetHelper.RunQueryNoReturn(s_log, query, conn);

                             return null;
                         }));
            goto Step_ScanSSCC;
          

            #endregion

        
            return;
        }

        #endregion

        #region HELPERS


        /// <summary>
        /// Gets the number of rows.
        /// </summary>
        /// <param name="ds">The ds.</param>
        /// <returns></returns>
        private int GetNumberOfRows(DataSet ds)
        {
            if (ds != null &&
                ds.Tables.Count > 0)
            {
                return ds.Tables[0].Rows.Count;
            }

            return 0;
        }


        #endregion

        #region QUERY

        private static class BuildQuery
        {
        

            /// <summary>
            /// Gets all sales deliveries.
            /// </summary>
            /// <returns></returns>
            internal static string GetAllSalesDeliveries(string status)
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.AppendFormat("SELECT DISTINCT {0}, {1}, {2}, {3}, {4}",
              SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxSalesDeliveryNoteTable.NAME, PmxSalesDeliveryNoteTable.Columns.CardCode.NAME, SalesDeliveryNoteDataSet.Columns.CUSTOMER_CODE.Name),
             SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxSalesDeliveryNoteTable.NAME, PmxSalesDeliveryNoteTable.Columns.CardName.NAME, SalesDeliveryNoteDataSet.Columns.CUSTOMER_NAME.Name),
             SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxSalesDeliveryNoteTable.NAME, PmxSalesDeliveryNoteTable.Columns.DocDueDate.NAME, SalesDeliveryNoteDataSet.Columns.DOC_DUE_DATE.Name),
              SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxSalesDeliveryNoteTable.NAME, PmxSalesDeliveryNoteTable.Columns.DocEntry.NAME, SalesDeliveryNoteDataSet.Columns.DOC_ENTRY.Name),
              SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxSalesDeliveryNoteTable.NAME, PmxSalesDeliveryNoteTable.Columns.DocNum.NAME, SalesDeliveryNoteDataSet.Columns.DOC_NUM.Name)
             );

                strBuilder.AppendFormat("\r\n FROM {0}",
             SqlCommandHelper.EscapeTableName(PmxSalesDeliveryNoteTable.NAME));

                strBuilder.AppendFormat("\r\n INNER JOIN {0} ON {1} = {2}",
                    SqlCommandHelper.EscapeTableName(PmxSalesDeliveryNoteLineTable.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxSalesDeliveryNoteTable.NAME, PmxSalesDeliveryNoteTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxSalesDeliveryNoteLineTable.NAME, PmxSalesDeliveryNoteLineTable.Columns.DocEntry.NAME));

            
                strBuilder.AppendFormat("\r\n INNER JOIN {0} ON {1} = {2} AND {3} = {4}",
                    SqlCommandHelper.EscapeTableName(PmxDeliveredSSCCRRTable.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxSalesDeliveryNoteTable.NAME, PmxSalesDeliveryNoteTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxDeliveredSSCCRRTable.NAME, PmxDeliveredSSCCRRTable.Columns.BaseEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxDeliveredSSCCRRTable.NAME, PmxDeliveredSSCCRRTable.Columns.Status.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(status, CultureInfo.InvariantCulture));

                //strBuilder.AppendFormat("\r\n WHERE {0} = {1}",
                //    SqlCommandHelper.EscapeTableColumnName(PmxSalesDeliveryNoteLineTable.NAME, PmxSalesDeliveryNoteLineTable.Columns.LineStatus.NAME),
                //    SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));

                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets all SSC cs.
            /// </summary>
            /// <param name="docEntry">The document entry.</param>
            /// <param name="status">The status.</param>
            /// <returns></returns>
            /// <exception cref="System.NotImplementedException"></exception>
            internal static string GetAllSSCCs(int docEntry, string status)
            {
                StringBuilder strBuilder = new StringBuilder();

                strBuilder.AppendFormat("SELECT {0}, {1}",
                     SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxDeliveredSSCCRRTable.NAME, PmxDeliveredSSCCRRTable.Columns.LUID.NAME, SSCCDataSet.Columns.LUID.Name),
              SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxDeliveredSSCCRRTable.NAME, PmxDeliveredSSCCRRTable.Columns.SSCC.NAME, SSCCDataSet.Columns.SSCC.Name)
             );


                strBuilder.Append(", ");
                SqlCommandHelper.AddStringNullColumnWithName(strBuilder, SSCCDataSet.Columns.PMX_WAREHOUSE_CODE.Name);
                strBuilder.Append(", ");
                SqlCommandHelper.AddStringNullColumnWithName(strBuilder, SSCCDataSet.Columns.PMX_WAREHOUSE_NAME.Name);
                strBuilder.Append(", ");
                SqlCommandHelper.AddStringNullColumnWithName(strBuilder, SSCCDataSet.Columns.STORAGE_LOCATION_CODE.Name);

                strBuilder.AppendFormat("\r\n FROM {0}",
             SqlCommandHelper.EscapeTableName(PmxDeliveredSSCCRRTable.NAME));

                strBuilder.Append(" WHERE ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxDeliveredSSCCRRTable.NAME, PmxDeliveredSSCCRRTable.Columns.BaseEntry.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(docEntry, CultureInfo.InvariantCulture));

                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxDeliveredSSCCRRTable.NAME, PmxDeliveredSSCCRRTable.Columns.Status.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(status, CultureInfo.InvariantCulture));



                return strBuilder.ToString();
               
            }

            /// <summary>
            /// Updates the delivered SSCC.
            /// </summary>
            /// <param name="luid">The luid.</param>
            /// <param name="docEntry">The document entry.</param>
            /// <param name="objectType">Type of the object.</param>
            /// <param name="status">The status.</param>
            /// <returns></returns>
            internal static string UpdateDeliveredSSCC(int luid, int docEntry, string objectType, string status)
            {
                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("UPDATE ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxDeliveredSSCCRRTable.NAME));

                strBuilder.Append(" SET ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxDeliveredSSCCRRTable.NAME, PmxDeliveredSSCCRRTable.Columns.Status.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(status, CultureInfo.InvariantCulture));


                strBuilder.Append(" WHERE ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxDeliveredSSCCRRTable.NAME, PmxDeliveredSSCCRRTable.Columns.LUID.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(luid, CultureInfo.InvariantCulture));

                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxDeliveredSSCCRRTable.NAME, PmxDeliveredSSCCRRTable.Columns.BaseType.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(objectType, CultureInfo.InvariantCulture));

                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxDeliveredSSCCRRTable.NAME, PmxDeliveredSSCCRRTable.Columns.BaseEntry.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(docEntry, CultureInfo.InvariantCulture));

                return strBuilder.ToString();
            }
        }

        #endregion

    }
}
