﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Data.Common;
using System.Globalization;
using System.Reflection;
using System.Text;
using System.Text.RegularExpressions;
using System.Transactions;
using Produmex.Foundation;
using Produmex.Foundation.Barcode;
using Produmex.Foundation.Data;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Convertors;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Generators.BarcodeController;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.GS1;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.SlimScreen.WinGui;
using Produmex.Foundation.Utilities;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Data.ViewObjects.Definitions.Views;
using Produmex.Foundation.Wwf.Sbo.DataObjects;
using Produmex.Foundation.Messages;
using Produmex.Foundation.Data.DbClient;
using Produmex.Sbo.Logex.Redrock.BusinessObjects.Definitions.Tables;

// PmxWorkflowExecutionTypes.SUB_FLOW
namespace Produmex.Sbo.Logex.WorkflowScripts
{
    /// <summary>
    /// v5
    /// </summary>
    public class WorkflowScript_TransferSSCCSalesOrderScript : WorkflowInstanceScriptBase
    {
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        // Input parameters
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;





        // Sub-flows
        // <none>



        /// <summary>
        /// Initializes a new instance of the <see cref="WorkflowScript_SelectProductFromStockScript" /> class.
        /// </summary>
        /// <param name="parent">The parent.</param>
        /// <param name="factory">The factory.</param>
        public WorkflowScript_TransferSSCCSalesOrderScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {
        }


        #region WorkflowInstanceScriptBase Members

        /// <summary>
        /// Executes this instance.
        /// </summary>
        protected override void Execute()
        {
            // Parameters in scope
            Session session = (Session)GetScopeParameter("Session");
            ISboProviderService sboProviderService = (ISboProviderService)GetScopeParameter("<WwfService>ISboService");

            string deviceID = session.DeviceInstanceId;
            string query = null;
            string initialErrorKey = null;
            //  SboUserInfo userInfo = sboProviderService.GetCurrentUser();
            Message msg = null;

            bool goBack = false;
            bool allowGoBack = true;
            bool isReceiving = false;
            bool isNew = false;
            bool isNewNoBarCode = false;
            bool askReprint = false;
            bool showMessage = false;

            DataRow[] selectedRow = null;

            DataSet dsSalesOrders = null;
            DataSet dsSSCC = null;
            DataSet dsWarehouses = null;
            DataSet dsZoneTypes = null;

            int luid = 0;
            int docEntry = 0;
            int baseDocNum = 0;
            int numberOfReports = 0;

            string supplierLogisticUnitID = null;
            string whsCode = null;
            string whsName = null;
            string zoneType = null;
            string buttonKeySelectTaskBeforeOrdering = null;
            string enteredSupplierPalletNumber = null;

            BarcodeContainer barCode = null;

        Step_SelectTask:

            #region SELECT TASK

            isReceiving = false;
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectButtonScreen),
                  DefaultCultureInfo.Get(), BuildParamCollection(
             "InitialErrorKey", initialErrorKey,
             "TitleKey", "MSG_TITLE_SELECT_TASK",
             "AllowedButtons", new string[] { "MSG_BUTTON_RR_RECEIVING", "MSG_BUTTON_RR_SHIPPING" },
             "ShowStopButton", false
             ));
            msg = WaitForMessage();

            initialErrorKey = null;

            if (msg.Name.EndsWith(".ButtonClicked"))
            {
                buttonKeySelectTaskBeforeOrdering = ExtractParameter<string>(msg.Parameters, "buttonKey");

                if (buttonKeySelectTaskBeforeOrdering == "MSG_BUTTON_RR_RECEIVING")
                {
                    isReceiving = true;
                }
                else if (buttonKeySelectTaskBeforeOrdering == "MSG_BUTTON_RR_SHIPPING")
                {
                    isReceiving = false;
                }
                goto Step_SelectSalesOrders;

            }
            else
            {
                return;
            }

            #endregion

        Step_SelectSalesOrders:

            #region SELECT SALES ORDER

            query = BuildQuery.GetAllSalesOrders();//(isReceiving ? );
            dsSalesOrders = sboProviderService.RunView(false, null, null, query);

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Shipping.ISelectSalesOrderScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                "InitialErrorKey", initialErrorKey,
                "TitleKey", "MSG_TITLE_SELECT_SALES_ORDER",
                "SalesOrderDS", dsSalesOrders
          ));

            msg = WaitForMessage();

            initialErrorKey = null;
            goBack = false;

            if (msg.Name.EndsWith(".SalesOrderSelected"))
            {
                docEntry = ExtractParameter<int?>(msg.Parameters, "docEntry").Value;

                selectedRow = dsSalesOrders.Tables[0].Select(string.Format("{0} = {1}", SalesOrderDataSet.Columns.DOC_ENTRY.Name, docEntry));

                baseDocNum = (Convert.ToInt32(selectedRow[0][SalesOrderDataSet.Columns.DOC_NUM.Name]));


            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
                goBack = true;
                goto Step_SelectTask;
            }
            else if (msg.Name.EndsWith(".RefreshRequested"))
            {
                goto Step_SelectSalesOrders;
            }

            goto Step_SelectEnterSSCCTask;

            #endregion


        Step_SelectEnterSSCCTask:

            #region SELECT ENTER SSCC TASK
            isNew = false;
            isNewNoBarCode = false;
            query = BuildQuery.GetAllSSCCs(docEntry, isReceiving ? PmxLUIDStatusColumnValuesRR.InTransit.VALUE : PmxLUIDStatusColumnValuesRR.Received.VALUE);
            dsSSCC = sboProviderService.RunView(true, null, null, query);

            if (GetNumberOfRows(dsSSCC) == 0 && showMessage)
            {
                showMessage = false;
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                                            DefaultCultureInfo.Get(), BuildParamCollection(
                                                "InitialErrorKey", initialErrorKey,
                                                "MessageKey", "MSG_RR_NOTHING_NEEDS_RECEPTION",
                                                "ShowButton", true));

                msg = WaitForMessage();
                if (!isReceiving)
                {
                    goto Step_SelectSalesOrders;
                }
            }

            string buttonkeySelectExistingPalletnr = "MSG_TITLE_RR_SELECT_EXISTING_SSCC";
            string buttonkeyEnterNewPalletNr = "MSG_TITLE_RR_ENTER_NEW_SSCC";

            if (!isReceiving)
            {
                if (goBack)
                {
                    goto Step_SelectSalesOrders;
                }
                goto Step_SelectSSCC;
            }

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectButtonScreen),
                  DefaultCultureInfo.Get(), BuildParamCollection(
             "InitialErrorKey", initialErrorKey,
             "TitleKey", "MSG_TITLE_SELECT_TASK",
             "AllowedButtons", new string[] { buttonkeySelectExistingPalletnr, buttonkeyEnterNewPalletNr },
             "ShowStopButton", false
             ));
            msg = WaitForMessage();

            initialErrorKey = null;

            if (msg.Name.EndsWith(".ButtonClicked"))
            {
                buttonKeySelectTaskBeforeOrdering = ExtractParameter<string>(msg.Parameters, "buttonKey");

                if (buttonKeySelectTaskBeforeOrdering == buttonkeySelectExistingPalletnr)
                {
                    goto Step_SelectSSCC;
                }
                else if (buttonKeySelectTaskBeforeOrdering == buttonkeyEnterNewPalletNr)
                {
                    goto Step_EnterSSCC;
                }

            }
            else
            {
                goto Step_SelectSalesOrders;
            }

            #endregion

        Step_SelectSSCC:

            #region SELECT SSCC

            query = BuildQuery.GetAllSSCCs(docEntry, isReceiving ? PmxLUIDStatusColumnValuesRR.InTransit.VALUE : PmxLUIDStatusColumnValuesRR.Received.VALUE);
            dsSSCC = sboProviderService.RunView(true, null, null, query);

            enteredSupplierPalletNumber = null;

            //if (GetNumberOfRows(dsSSCC) == 0)
            //{
            //    goto Step_SelectSalesOrders;
            //}
            //session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IIdentifySSCCListScreen),
            //                   DefaultCultureInfo.Get(), BuildParamCollection(
            //                   "InitialErrorKey", initialErrorKey,
            //                       "TitleKey", "MSG_TITLE_RR_SELECT_SSCC",
            //                       "SSCCDS", dsSSCC,
            //                       "ButtonKey", isReceiving ? "MSG_BUTTON_MAKE_NEW_SSCC" : null
            //                       ));
            //msg = WaitForMessage();
            //initialErrorKey = null;

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectEnterStringValueScreen),
           DefaultCultureInfo.Get(), BuildParamCollection(
           "InitialErrorKey", initialErrorKey,
           "TitleKey", "MSG_TITLE_RR_SELECT_SSCC",
           "AllowToGoBack", allowGoBack,
              "StringValueDS", dsSSCC
            ));
            initialErrorKey = null;
            msg = WaitForMessage();
            goBack = false;
            allowGoBack = true;

            if (msg.Name.EndsWith(".StringValueSelected"))
            {
                enteredSupplierPalletNumber = ExtractParameter<string>(msg.Parameters, StringValueDataSet.Columns.STRING_VALUE.Name);
                selectedRow = dsSSCC.Tables[0].Select(string.Format("{0} = {1}", StringValueDataSet.Columns.STRING_VALUE.Name, enteredSupplierPalletNumber));

                if (selectedRow.Length == 0)
                {
                    initialErrorKey = "MSG_RR_ERROR_SSCC_NOT_FOUND";
                    enteredSupplierPalletNumber = null;
                    selectedRow = null;
                    goto Step_SelectSSCC;
                }

                luid = Convert.ToInt32(selectedRow[0][PmxReservedLUIDRRTable.Columns.LUID.NAME]);

                //  isNew = false;
                if (isReceiving)
                {
                    goto Step_SelectWarehouse;
                }
                goto Step_SetData;

            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
                goBack = true;
                goto Step_SelectEnterSSCCTask;
            }

            #endregion

        Step_EnterSSCC:

            #region ENTER NEW SSCC

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IIdentifyGS1DataScreen),
                    this.DefaultCultureInfo, BuildParamCollection(
                    "InitialErrorKey", initialErrorKey,
                    "TitleKey", "MSG_TITLE_IDENTIFY_SSCC",
                    "ForceGS1_128", false,
                    "AllowedButtons", new string[]{"MSG_BUTTON_RR_CREATE_PALLET_NO_BARCODE"}
                    ));
            initialErrorKey = null;
            msg = WaitForMessage();


            //session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterStringValueScreen),
            //   DefaultCultureInfo.Get(), BuildParamCollection(
            //       "InitialErrorKey", initialErrorKey,
            //       "TitleKey", "MSG_TITLE_IDENTIFY_SSCC",
            //       "ForceDataEntry", true,
            //       "AllowToGoBack", true,
            //       "ButtonKey", "MSG_BUTTON_RR_CREATE_PALLET_NO_BARCODE"
            //       ));
            //msg = WaitForMessage();
            //goBack = false;
            //// reset error
            //initialErrorKey = null;

            isNew = true;
            //if (msg.Name.EndsWith(".StringEntered"))
            if (msg.Name.EndsWith(".DataEntered"))
            {
                //enteredSupplierPalletNumber = ExtractParameter<string>(msg.Parameters, "stringValue");
                barCode = ExtractParameter<BarcodeContainer>(msg.Parameters, "data");

                if (barCode != null &&
                    !string.IsNullOrEmpty(barCode.GS1DataObject.Sscc))
                {
                    enteredSupplierPalletNumber = barCode.GS1DataObject.Sscc;
                    goto Step_SelectWarehouse;
                }
                else if (barCode != null &&
                     !string.IsNullOrEmpty(ValidateSupplierPalletNumber(barCode)))
                {
                    enteredSupplierPalletNumber = barCode.FreeBarcode;
                    goto Step_SelectWarehouse;
                }
                else
                {
                    initialErrorKey = "MSG_ERROR_INVALID_PALLET_NR";
                     enteredSupplierPalletNumber = null;
                    selectedRow = null;
                    goto Step_EnterSSCC;
                }

                //check if SSCC exists
                selectedRow = dsSSCC.Tables[0].Select(string.Format("{0} = {1}", StringValueDataSet.Columns.STRING_VALUE.Name, enteredSupplierPalletNumber));

                if (selectedRow.Length > 0)
                {
                    //SSCC not valid
                    initialErrorKey = "MSG_ERROR_RCI_NO_NEW_PALLET_NR";
                    enteredSupplierPalletNumber = null;
                    selectedRow = null;
                    goto Step_EnterSSCC;
                }
                
                ////get warehouse
                //whsInfo = sboProviderService.InvokeMethodWithDbConnection<PmxOseWarehouseInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOseWarehouseInfo>(ExecuteGetPmxWarehouseCode), dsProductsOnSSCC.Tables[0].Rows[0][PmxInventoryTotalTable.Columns.StorLocCode.NAME] as string);

              
                goto Step_SelectWarehouse;
            }
            else if (msg.Name.EndsWith(".ButtonClicked"))
            {
                isNewNoBarCode = true;
                
                goto Step_SelectWarehouse;
            }
            else
            {
                goto Step_SelectEnterSSCCTask;
            }

            #endregion


        Step_SelectWarehouse:

            #region SELECT WAREHOUSE

            query = BuildQuery.GetWarehouses();
            dsWarehouses = sboProviderService.RunView(true, null, null, query);
            whsCode = null;
            whsName = null;

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectEnterStringValueScreen),
           DefaultCultureInfo.Get(), BuildParamCollection(
           "InitialErrorKey", initialErrorKey,
           "TitleKey", "MSG_TITLE_RR_SELECT_WAREHOUSE",
           "AllowToGoBack", allowGoBack,
           "StringValueDS", dsWarehouses));
            initialErrorKey = null;
            msg = WaitForMessage();
            goBack = false;
            allowGoBack = true;

            if (msg.Name.EndsWith(".StringValueSelected"))
            {
                whsName = ExtractParameter<string>(msg.Parameters, "stringValue");
                string filter = string.Format("{0} = '{1}'", StringValueDataSet.Columns.STRING_VALUE.Name, whsName);
                //SqlCommandHelper.FormatAndEscapeColumnValue(whsName, CultureInfo.InvariantCulture));
                selectedRow = dsWarehouses.Tables[0].Select(filter);
                whsCode = selectedRow[0][PmxOseWarehouseTable.Columns.Code.NAME] as string;

                goto Step_SelectZoneType;
            }
            else
            {
                if (isReceiving)
                {
                    goto Step_SelectSSCC;
                }
                goto Step_SelectSalesOrders;
            }

            #endregion


        Step_SelectZoneType:

            #region SELECT ZONE TYPE

            query = BuildQuery.GetZoneTypes(whsCode);
            dsZoneTypes = sboProviderService.RunView(true, null, null, query);
            zoneType = null;

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectEnterStringValueScreen),
           DefaultCultureInfo.Get(), BuildParamCollection(
           "InitialErrorKey", initialErrorKey,
           "TitleKey", "MSG_TITLE_RR_SELECT_ZONETYPES",
           "AllowToGoBack", allowGoBack,
           "StringValueDS", dsZoneTypes));
            initialErrorKey = null;
            msg = WaitForMessage();
            goBack = false;
            allowGoBack = true;

            if (msg.Name.EndsWith(".StringValueSelected"))
            {
                zoneType = ExtractParameter<string>(msg.Parameters, "stringValue");
                goto Step_SetData;
            }
            else
            {
                goto Step_SelectWarehouse;
            }


            #endregion


        Step_SetData:

            #region SET DATA

            if (isReceiving && isNew)
            {
                if (isNewNoBarCode)
                {
                    LogisticUnitInfo logUnitInfo = sboProviderService.InvokeMethodWithDbConnection<LogisticUnitInfo>(false, false, null, null, new DelegateWithPmxDbConnection<LogisticUnitInfo>(ExecuteGenerateNewLuidWithSSCC), enteredSupplierPalletNumber);
                    enteredSupplierPalletNumber = logUnitInfo.SSCC;
                    luid = logUnitInfo.Luid;
                }
                else
                {
                    luid = sboProviderService.InvokeMethodWithDbConnection<int>(false, false, null, null, new DelegateWithPmxDbConnection<int>(ExecuteGenerateLuid), enteredSupplierPalletNumber);
                }

                sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, new DelegateWithPmxDbConnection<object>(
                       delegate(PmxDbConnection conn, object[] parameters)
                       {
                           query = BuildQuery.InsertReservedLUID(docEntry, baseDocNum, SboObjectTypes.SalesOrder, luid, enteredSupplierPalletNumber,
                               PmxLUIDStatusColumnValuesRR.Received.VALUE, whsName, zoneType,
                              conn.InternalKeySboUser);
                           SboRecordsetHelper.RunQueryNoReturn(s_log, query, conn);

                           return null;
                       }));
                goto Step_PrintLabel;

            }
            else
            {


                sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, new DelegateWithPmxDbConnection<object>(
                         delegate(PmxDbConnection conn, object[] parameters)
                         {
                             query = BuildQuery.UpdateReservedLUID(docEntry, SboObjectTypes.SalesOrder, luid, isReceiving ? PmxLUIDStatusColumnValuesRR.Received.VALUE : PmxLUIDStatusColumnValuesRR.InTransit.VALUE, whsName);
                             SboRecordsetHelper.RunQueryNoReturn(s_log, query, conn);

                             return null;
                         }));
                showMessage = true;
                goto Step_SelectEnterSSCCTask;
            }


            #endregion


        Step_PrintLabel:

            #region PRINT LABEL

            // Print LABEL
            if (isNew)
            {
                bool proceed = false;
                while (!proceed)
                {
                    numberOfReports = 0;

                    try
                    {
                        string destinationLocationCode = null;

                        //trigger event
                        numberOfReports += sboProviderService.InvokeMethodWithDbConnection<int>(false, true, null, null, delegate(PmxDbConnection conn, object[] parameters)
                        {
                            PmxPrintReportProvider printProv = new PmxPrintReportProvider(conn);
                            return printProv.PrintReportsForEvent(PmxPrintReportEventType.CustomPrintReportEvent1, destinationLocationCode, session.DeviceInstanceId, luid, out askReprint);
                        });
                    }
                    catch (Produmex.Foundation.ProdumexException ex)
                    {
                        s_log.Error(ex.ToString());
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "InitialErrorKey", initialErrorKey,
                                "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                                "MessageAddendum", ex.Message,
                                "PmxError", ex.Error,
                                "ShowButton", true));

                        msg = WaitForMessage();
                    }
                    catch (Exception ex)
                    {
                        s_log.Error(ex.ToString());
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "InitialErrorKey", initialErrorKey,
                                "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                                "MessageAddendum", ex.Message,
                                "PmxError", Produmex.Sbo.Logex.Data.ProdumexErrors.UnexpectedError(s_log, ex),
                                "ShowButton", true));

                        msg = WaitForMessage();
                    }

                    if (numberOfReports > 0 && askReprint)
                    {
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "MessageKey", "MSG_QUESTION_REPRINT_LABEL",
                                "TitleKey", "MSG_TITLE_REPRINT_LABEL"
                                ));

                        msg = WaitForMessage();

                        if (msg.Name.EndsWith(".No"))
                        {
                            proceed = true;
                        }

                    }
                    else
                    {
                        proceed = true;
                    }

                }


            }
            showMessage = true;
            goto Step_SelectEnterSSCCTask;

            #endregion

            // return;


        }

       
        #endregion

        #region HELPERS

        /// <summary>
        /// Executes the get item info.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxItemInfo ExecuteGetItemInfo(PmxDbConnection conn, object[] parameters)
        {
            PmxItemAllConnectionsProvider prov = new PmxItemAllConnectionsProvider(conn);
            return prov.GetCachedItemInfo((string)parameters[0]);
        }

        /// <summary>
        /// Executes the generate luid.
        /// </summary>
        /// <param name="conn">The connection.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static int ExecuteGenerateLuid(PmxDbConnection conn, object[] parameters)
        {
            PmxLogisticUnitIDProvider logUnitProv = new PmxLogisticUnitIDProvider(conn);
            //string sscc = (string)parameters[0];
            string supplierPalletNumber = (string)parameters[0];
            int logisticUnitId = -1;

            logisticUnitId = logUnitProv.GenerateNewLogisticUnit(supplierPalletNumber);

            return logisticUnitId;
        }

        /// <summary>
        /// Executes the generate new luid.
        /// </summary>
        /// <param name="conn">The connection.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static LogisticUnitInfo ExecuteGenerateNewLuidWithSSCC(PmxDbConnection conn, object[] parameters)
        {
            PmxLogisticUnitIDProvider logUnitProv = new PmxLogisticUnitIDProvider(conn);
            string sscc;
            int luid = logUnitProv.GenerateNewLogisticUnitSSCC(null, out sscc);

            return new LogisticUnitInfo() { Luid = luid, SSCC = sscc };
        }

        /// <summary>
        /// Gets the number of rows.
        /// </summary>
        /// <param name="ds">The ds.</param>
        /// <returns></returns>
        private int GetNumberOfRows(DataSet ds)
        {
            if (ds != null &&
                ds.Tables.Count > 0)
            {
                return ds.Tables[0].Rows.Count;
            }

            return 0;
        }

        /// <summary>
        /// Validates the supplier pallet number.
        /// </summary>
        /// <param name="barCode">The bar code.</param>
        /// <returns></returns>
        private string ValidateSupplierPalletNumber(BarcodeContainer barCode)
        {
            string palletNumber = barCode.FreeBarcode;
            if (!string.IsNullOrEmpty(palletNumber)
                && GS1DataElementValidator.MatchesRegex(palletNumber, "[0-9]$")
                ) // Will match all sequences of 18 numbers
            {
                return palletNumber;
            }
            return null;
        }


        #endregion

        #region QUERY

        private static class BuildQuery
        {

            /// <summary>
            /// Gets all sales orders.
            /// </summary>
            /// <param name="deviceID">The device identifier.</param>
            /// <param name="p">if set to <c>true</c> [p].</param>
            /// <returns></returns>
            public static string GetAllSalesOrders()
            {
                StringBuilder strBuilder = new StringBuilder();

                strBuilder.AppendFormat("SELECT DISTINCT {0}, {1}, {2}, {3}, {4}, {5}",
              SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxSalesOrderTable.NAME, PmxSalesOrderTable.Columns.CardCode.NAME, SalesOrderDataSet.Columns.CUSTOMER_CODE.Name),
             SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxSalesOrderTable.NAME, PmxSalesOrderTable.Columns.CardName.NAME, SalesOrderDataSet.Columns.CUSTOMER_NAME.Name),
             SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxSalesOrderTable.NAME, PmxSalesOrderTable.Columns.DocDueDate.NAME, SalesOrderDataSet.Columns.DOC_DUE_DATE.Name),
              SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxSalesOrderTable.NAME, PmxSalesOrderTable.Columns.DocEntry.NAME, SalesOrderDataSet.Columns.DOC_ENTRY.Name),
              SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxSalesOrderTable.NAME, PmxSalesOrderTable.Columns.DocNum.NAME, SalesOrderDataSet.Columns.DOC_NUM.Name),
              SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxPickListTypeTableRR.NAME, PmxPickListTypeTableRR.Columns.Name.NAME, SalesOrderDataSet.Columns.PICK_LIST_TYPE.Name)
             );

                strBuilder.AppendFormat("\r\n FROM {0}",
             SqlCommandHelper.EscapeTableName(PmxSalesOrderTable.NAME));

                strBuilder.AppendFormat("\r\n INNER JOIN {0} ON {1} = {2}",
                    SqlCommandHelper.EscapeTableName(PmxSalesOrderLineTable.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxSalesOrderTable.NAME, PmxSalesOrderTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxSalesOrderLineTable.NAME, PmxSalesOrderLineTable.Columns.DocEntry.NAME));


                strBuilder.AppendFormat("\r\n INNER JOIN {0} ON {1} = {2}",
                    SqlCommandHelper.EscapeTableName(SboWarehouseTable.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxSalesOrderLineTable.NAME, PmxSalesOrderLineTable.Columns.WarehouseCode.NAME),
                    SqlCommandHelper.EscapeTableColumnName(SboWarehouseTable.NAME, SboWarehouseTable.Columns.WarehouseCode.NAME));

                strBuilder.AppendFormat("\r\n INNER JOIN {0} ON {1} = {2}",
                    SqlCommandHelper.EscapeTableName(PmxPickListTypeTable.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxSalesOrderTable.NAME, PmxSalesOrderTable.Columns.PickListType.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxPickListTypeTable.NAME, PmxPickListTypeTable.Columns.Code.NAME));

                strBuilder.AppendFormat("\r\n WHERE {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxSalesOrderLineTable.NAME, PmxSalesOrderLineTable.Columns.LineStatus.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, CultureInfo.InvariantCulture));

                strBuilder.AppendFormat("\r\n AND {0} = {1}",
                 SqlCommandHelper.EscapeTableColumnName(PmxPickListTypeTable.NAME, PmxPickListTypeTableRR.Columns.UseInDropShipment.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));


                //strBuilder.AppendFormat("\r\n AND {0} = {1}",
                //  SqlCommandHelper.EscapeTableColumnName(SboWarehouseTable.NAME, SboWarehouseTable.Columns.DropShip.NAME),
                //  SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));



                //if (!String.IsNullOrEmpty(deviceID))
                //{
                //    strBuilder.Append("\r\n AND ( ");
                //    if (!String.IsNullOrEmpty(deviceID))
                //    {

                //        strBuilder.AppendFormat("{0} IN ( SELECT {1} FROM {2} WHERE {3} = {4} )",
                //            SqlCommandHelper.EscapeTableColumnName(PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME),
                //            SqlCommandHelper.EscapeTableColumnName(PmxThinClientWarehouseTable.NAME, PmxThinClientWarehouseTable.Columns.PmxWarehouseCode.NAME),
                //            SqlCommandHelper.EscapeTableName(PmxThinClientWarehouseTable.NAME),
                //            SqlCommandHelper.EscapeTableColumnName(PmxThinClientWarehouseTable.NAME, PmxThinClientWarehouseTable.Columns.ParentCode.NAME),
                //            SqlCommandHelper.FormatAndEscapeColumnValue(deviceID, CultureInfo.InvariantCulture));
                //    }
                //    strBuilder.Append(" )");
                //}

                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets all SSC cs.
            /// </summary>
            /// <returns></returns>
            /// <exception cref="System.NotImplementedException"></exception>
            public static string GetAllSSCCs(int docEntry, string status)
            {
                StringBuilder strBuilder = new StringBuilder();

                //   strBuilder.AppendFormat("SELECT {0}, {1}",
                //        SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxReservedLUIDRRTable.NAME, PmxReservedLUIDRRTable.Columns.LUID.NAME, SSCCDataSet.Columns.LUID.Name),
                // SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxReservedLUIDRRTable.NAME, PmxReservedLUIDRRTable.Columns.SupplierLogisticUnitID.NAME, SSCCDataSet.Columns.SSCC.Name)
                //);
                strBuilder.AppendFormat("SELECT {0}, {1}",
                  SqlCommandHelper.EscapeTableColumnName(PmxReservedLUIDRRTable.NAME, PmxReservedLUIDRRTable.Columns.LUID.NAME),
           SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxReservedLUIDRRTable.NAME, PmxReservedLUIDRRTable.Columns.SupplierLogisticUnitID.NAME, StringValueDataSet.Columns.STRING_VALUE.Name)

          );

                //strBuilder.Append(", ");
                //SqlCommandHelper.AddStringNullColumnWithName(strBuilder, SSCCDataSet.Columns.PMX_WAREHOUSE_CODE.Name);
                //strBuilder.Append(", ");
                //SqlCommandHelper.AddStringNullColumnWithName(strBuilder, SSCCDataSet.Columns.PMX_WAREHOUSE_NAME.Name);
                //strBuilder.Append(", ");
                //SqlCommandHelper.AddStringNullColumnWithName(strBuilder, SSCCDataSet.Columns.STORAGE_LOCATION_CODE.Name);

                strBuilder.AppendFormat("\r\n FROM {0}",
             SqlCommandHelper.EscapeTableName(PmxReservedLUIDRRTable.NAME));

                strBuilder.Append("\r\n WHERE ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxReservedLUIDRRTable.NAME, PmxReservedLUIDRRTable.Columns.BaseEntry.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(docEntry, CultureInfo.InvariantCulture));

                strBuilder.Append("\r\n AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxReservedLUIDRRTable.NAME, PmxReservedLUIDRRTable.Columns.Status.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(status, CultureInfo.InvariantCulture));

                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets the warehouses.
            /// </summary>
            /// <returns></returns>
            /// <exception cref="System.NotImplementedException"></exception>
            public static string GetWarehouses()
            {
                StringBuilder strBuilder = new StringBuilder();

                strBuilder.AppendFormat("SELECT {0}, {1}",
              SqlCommandHelper.EscapeTableColumnName(PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME),
              SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, StringValueDataSet.Columns.STRING_VALUE.Name)
             );

                strBuilder.AppendFormat("\r\n FROM {0}",
             SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));

                //if (!String.IsNullOrEmpty(deviceID))
                //{
                //    strBuilder.Append("\r\n WHERE ( ");
                //    if (!String.IsNullOrEmpty(deviceID))
                //    {

                //        strBuilder.AppendFormat("{0} IN ( SELECT {1} FROM {2} WHERE {3} = {4} )",
                //            SqlCommandHelper.EscapeTableColumnName(PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME),
                //            SqlCommandHelper.EscapeTableColumnName(PmxThinClientWarehouseTable.NAME, PmxThinClientWarehouseTable.Columns.PmxWarehouseCode.NAME),
                //            SqlCommandHelper.EscapeTableName(PmxThinClientWarehouseTable.NAME),
                //            SqlCommandHelper.EscapeTableColumnName(PmxThinClientWarehouseTable.NAME, PmxThinClientWarehouseTable.Columns.ParentCode.NAME),
                //            SqlCommandHelper.FormatAndEscapeColumnValue(deviceID, CultureInfo.InvariantCulture));
                //    }
                //    strBuilder.Append(" )");
                //}

                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets the zone types.
            /// </summary>
            /// <param name="whsCode">The WHS code.</param>
            /// <returns></returns>
            internal static string GetZoneTypes(string whsCode)
            {
                StringBuilder strBuilder = new StringBuilder();

                strBuilder.AppendFormat("SELECT {0}, {1}",
            SqlCommandHelper.EscapeTableColumnName(PmxZoneTypeTable.NAME, PmxZoneTypeTable.Columns.Code.NAME),
            SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxZoneTypeTable.NAME, PmxZoneTypeTable.Columns.Name.NAME, StringValueDataSet.Columns.STRING_VALUE.Name)
           );

                strBuilder.AppendFormat("\r\n FROM {0}",
             SqlCommandHelper.EscapeTableName(PmxZoneTypeTable.NAME));

                //          strBuilder.AppendFormat("\r\n INNER JOIN {0} ON {1} = {2}",
                //SqlCommandHelper.EscapeTableName(SboWarehouseTable.NAME),
                //SqlCommandHelper.EscapeTableColumnName(PmxSalesOrderLineTable.NAME, PmxSalesOrderLineTable.Columns.WarehouseCode.NAME),
                //SqlCommandHelper.EscapeTableColumnName(SboWarehouseTable.NAME, SboWarehouseTable.Columns.WarehouseCode.NAME));


                return strBuilder.ToString();
            }


            /// <summary>
            /// Inserts the reserved lui das shipped.
            /// </summary>
            /// <param name="docEntry">The document entry.</param>
            /// <param name="docNum">The document number.</param>
            /// <param name="objectType">Type of the object.</param>
            /// <param name="luid">The luid.</param>
            /// <param name="supplierLogisticUnitID">The supplier logistic unit identifier.</param>
            /// <param name="status">The status.</param>
            /// <param name="whsName">Name of the WHS.</param>
            /// <param name="zoneType">Type of the zone.</param>
            /// <returns></returns>
            internal static string InsertReservedLUID(int docEntry, int docNum, string objectType, int luid, string supplierLogisticUnitID, string status, string whsName, string zoneType, int userSign)
            {

                StringBuilder strBuilder = new StringBuilder();

                strBuilder.Append("INSERT INTO ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxReservedLUIDRRTable.NAME));
                strBuilder.Append("\r\n ( ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxReservedLUIDRRTable.Columns.Canceled.NAME));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxReservedLUIDRRTable.Columns.UserSign.NAME));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxReservedLUIDRRTable.Columns.CreateDateTime.NAME));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxReservedLUIDRRTable.Columns.UpdateDateTime.NAME));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxReservedLUIDRRTable.Columns.Version.NAME));
                strBuilder.Append(", ");

                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxReservedLUIDRRTable.Columns.BaseEntry.NAME));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxReservedLUIDRRTable.Columns.BaseDocNum.NAME));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxReservedLUIDRRTable.Columns.BaseType.NAME));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxReservedLUIDRRTable.Columns.LUID.NAME));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxReservedLUIDRRTable.Columns.SupplierLogisticUnitID.NAME));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxReservedLUIDRRTable.Columns.Status.NAME));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxReservedLUIDRRTable.Columns.ZoneType.NAME));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxReservedLUIDRRTable.Columns.Warehouse.NAME));


                strBuilder.Append(" )\r\n VALUES ( ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue("N", CultureInfo.InvariantCulture));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(userSign, CultureInfo.InvariantCulture));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(DateTime.Now, CultureInfo.InvariantCulture));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(DateTime.Now, CultureInfo.InvariantCulture));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(1, CultureInfo.InvariantCulture));
                strBuilder.Append(", ");


                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(docEntry, CultureInfo.InvariantCulture));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(docNum, CultureInfo.InvariantCulture));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(objectType, CultureInfo.InvariantCulture));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(luid, CultureInfo.InvariantCulture));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(supplierLogisticUnitID, CultureInfo.InvariantCulture));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(status, CultureInfo.InvariantCulture));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(zoneType, CultureInfo.InvariantCulture));
                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(whsName, CultureInfo.InvariantCulture));
                strBuilder.Append(" )");

                return strBuilder.ToString();
            }

            /// <summary>
            /// Updates the reserved lui das shipped.
            /// </summary>
            /// <param name="docEntry">The document entry.</param>
            /// <param name="objectType">Type of the object.</param>
            /// <param name="luid">The luid.</param>
            /// <param name="status">The status.</param>
            /// <returns></returns>
            internal static string UpdateReservedLUID(int docEntry, string objectType, int luid, string status, string whsName)
            {

                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("UPDATE ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxReservedLUIDRRTable.NAME));

                strBuilder.Append(" SET ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxReservedLUIDRRTable.NAME, PmxReservedLUIDRRTable.Columns.Status.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(status, CultureInfo.InvariantCulture));

                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxReservedLUIDRRTable.NAME, PmxReservedLUIDRRTable.Columns.Warehouse.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(whsName, CultureInfo.InvariantCulture));

                strBuilder.Append(" WHERE ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxReservedLUIDRRTable.NAME, PmxReservedLUIDRRTable.Columns.LUID.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(luid, CultureInfo.InvariantCulture));

                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxReservedLUIDRRTable.NAME, PmxReservedLUIDRRTable.Columns.BaseType.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(objectType, CultureInfo.InvariantCulture));

                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxReservedLUIDRRTable.NAME, PmxReservedLUIDRRTable.Columns.BaseEntry.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(docEntry, CultureInfo.InvariantCulture));

                return strBuilder.ToString();
            }
        }

        /// <summary>
        /// 
        /// </summary>
        private class LogisticUnitInfo
        {
            /// <summary>
            /// Gets or sets the luid.
            /// </summary>
            /// <value>
            /// The luid.
            /// </value>
            public int Luid { get; set; }
            /// <summary>
            /// Gets or sets the SSCC.
            /// </summary>
            /// <value>
            /// The SSCC.
            /// </value>
            public string SSCC { get; set; }

            ///// <summary>
            ///// Gets or sets the supplier logistic unit identifier.
            ///// </summary>
            ///// <value>
            ///// The supplier logistic unit identifier.
            ///// </value>
            //public string SupplierLogisticUnitID { get; set; }
        }
        #endregion

    }
}
